var callSoapMethod, extractFieldsFromWSDL, getFields, getTableData, getTableList, getTableName, parseWebiTableXml, soap, util;

soap = require('soap');

util = require('./util');

parseWebiTableXml = require('./table_sax_parser');

callSoapMethod = function(client, methodName, credentials, options, callback) {
  if (options == null) {
    options = {};
  }
  options.login = credentials.username;
  options.password = credentials.password;
  return client[methodName]([options], callback);
};

extractFieldsFromWSDL = function(client, credentials, tableName, callback) {
  var field, fields, ref, typeDesc;
  fields = [];
  if (util.getServiceType(client.wsdl) === util.QAWS) {
    ref = client.wsdl.definitions.descriptions.types.Row;
    for (field in ref) {
      typeDesc = ref[field];
      if (field !== 'targetNSAlias' && field !== 'targetNamespace') {
        fields.push({
          name: field,
          type: util.getType(typeDesc)
        });
      }
    }
    return callback(null, fields);
  } else {
    return callSoapMethod(client, util.getMethodName(tableName), credentials, {
      startRow: 1,
      endRow: 1
    }, function(err, results, raw) {
      var errorMessage, ref1;
      if ((results != null ? (ref1 = results.headers) != null ? ref1.row : void 0 : void 0) == null) {
        errorMessage = "Could not get records from soap.";
        if ((results != null ? results.message : void 0) != null) {
          errorMessage += results.message;
        }
        err = new Error(errorMessage);
      }
      if (err == null) {
        fields = parseWebiTableXml(raw).fields.map(function(e) {
          return {
            name: e.name,
            type: util.getType(e.type)
          };
        });
      }
      return callback(err, fields);
    });
  }
};

getTableList = function(wsdlUrl, callback) {
  return soap.createClient(wsdlUrl, function(err, client) {
    var method, methodName, ref, reportName, reports, serviceType, tableName, tables;
    if (err == null) {
      tables = [];
      ref = client.describe();
      for (reportName in ref) {
        reports = ref[reportName];
        for (serviceType in reports) {
          method = reports[serviceType];
          for (methodName in method) {
            tableName = util.methodToTable(methodName);
            if (tableName != null) {
              tables.push(tableName);
            }
          }
        }
      }
      return callback(err, tables);
    } else {
      return callback(err, null);
    }
  });
};

getTableName = function(wsdlUrl, callback) {
  return soap.createClient(wsdlUrl, function(err, client) {
    var ref, tableName, tables;
    if (err == null) {
      ref = client.describe();
      for (tableName in ref) {
        tables = ref[tableName];
        callback(err, tableName);
        return;
      }
      return callback(new Error("Could not find table name in wsdl"), null);
    } else {
      return callback(err, null);
    }
  });
};

getFields = function(wsdlUrl, credentials, tableName, callback) {
  if (wsdlUrl == null) {
    callback(new Error("Invalid config. No WSDL set."));
    return;
  }
  return soap.createClient(wsdlUrl, function(err, client) {
    if (err == null) {
      return extractFieldsFromWSDL(client, credentials, tableName, callback);
    } else {
      return callback(err, null);
    }
  });
};

getTableData = function(wsdlUrl, credentials, tableName, options, callback) {
  return getFields(wsdlUrl, credentials, tableName, function(err, fields) {
    if (err == null) {
      return soap.createClient(wsdlUrl, function(err, client) {
        var soapOptions;
        if (err == null) {
          soapOptions = {};
          if (((options != null ? options.Limit : void 0) != null) > 0) {
            soapOptions.startRow = 1;
            soapOptions.endRow = options.Limit;
          }
          return callSoapMethod(client, util.getMethodName(tableName, client), credentials, soapOptions, function(err, results, raw) {
            var data;
            if (err == null) {
              data = results.table.row;
              if (util.getServiceType(client.wsdl) === util.QAWS) {
                data = util.transformQAWSToObjectArray(fields, data);
              } else {
                data = parseWebiTableXml(raw).data;
              }
            }
            return callback(err, data);
          });
        } else {
          return callback(err, null);
        }
      });
    } else {
      return callback(err, null);
    }
  });
};

module.exports = {
  getTableList: getTableList,
  getFields: getFields,
  getTableData: getTableData
};

//# sourceMappingURL=index.js.map
