var generateResultParser, sax;

sax = require("sax");

generateResultParser = function(data) {
  var columnTypes, currentRow, fieldIdx, fieldName, headerFields, isInCell, isInHeader, isInTable, o, output, parser, row, rows, state, strict, typesOut;
  state = {};
  strict = true;
  parser = sax.parser(strict);
  isInTable = false;
  isInCell = true;
  rows = [];
  currentRow = [];
  columnTypes = [];
  isInHeader = false;
  headerFields = [];
  parser.onerror = function(e) {
    return console.error("ERROR:", e);
  };
  parser.ontext = function(t) {
    if (isInHeader) {
      headerFields.push(t);
    }
    if (isInCell) {
      return currentRow.push(t);
    }
  };
  parser.onopentag = function(node) {
    switch (node.name) {
      case 'table':
        return isInTable = true;
      case 'headers':
        return isInHeader = true;
      case 'row':
        currentRow = [];
        return columnTypes = [];
      case 'cell':
        if (!isInTable) {
          break;
        }
        columnTypes.push(node.attributes['xsi:type']);
        if (node.attributes['xsi:nil']) {
          return currentRow.push(null);
        } else {
          return isInCell = true;
        }
    }
  };
  parser.onclosetag = function(name) {
    switch (name) {
      case 'table':
        return isInTable = false;
      case 'headers':
        return isInHeader = false;
      case 'row':
        if (!isInTable) {
          break;
        }
        rows.push(currentRow);
        return currentRow = [];
      case 'cell':
        return isInCell = false;
    }
  };
  parser.write(data).close();
  output = (function() {
    var i, j, len, len1, results;
    results = [];
    for (i = 0, len = rows.length; i < len; i++) {
      row = rows[i];
      o = {};
      for (fieldIdx = j = 0, len1 = headerFields.length; j < len1; fieldIdx = ++j) {
        fieldName = headerFields[fieldIdx];
        o[fieldName] = row[fieldIdx];
      }
      results.push(o);
    }
    return results;
  })();
  typesOut = (function() {
    var i, len, results;
    results = [];
    for (fieldIdx = i = 0, len = headerFields.length; i < len; fieldIdx = ++i) {
      fieldName = headerFields[fieldIdx];
      results.push({
        name: fieldName,
        type: columnTypes[fieldIdx]
      });
    }
    return results;
  })();
  return {
    data: output,
    fields: typesOut
  };
};

module.exports = generateResultParser;

//# sourceMappingURL=table_sax_parser.js.map
